//http://homofaciens.de/technics-physical-computing-set01_en.htm

//Parts of the code from: http://www.learningaboutelectronics.com/Articles/Arduino-keypad-circuit.php

//Switch a relay after code input through 4x4 button keypad

//Pins when looking at the FRONT side of the push button matrix
//From left to right:
//Pin 1 to Arduino pin 9
//Pin 2 to Arduino pin 8
//Pin 3 to Arduino pin 7
//Pin 4 to Arduino pin 6
//Pin 5 to Arduino pin 5
//Pin 6 to Arduino pin 4
//Pin 7 to Arduino pin 3
//Pin 8 to Arduino pin 2

//Relay (+) to Arduino +5V
//Relay (-) to Arduino GND
//Relay S to Arduino A2

//Connect one side of the push button to GND of the Arduino
//Connect the second side of the push button to a 10 kiloohm or 1 kiloohm resistor
//Connect the second side of the resistor to pin A3 of the Arduino

//Connect kathode (- short pin, flat side of housing) of LED to GND of Arduino
//Connect anode (+ long pin, round side of housing) to 220 Ohm resistor
//Connect second end of 220 Ohm resistor to pin A1 of Arduino



#include <Keypad.h>

#define RELAY_PIN           A2
#define ON_BUTTON_PIN       A3
#define CODE_WRONG_LED_PIN  A1

#define KEYPAD_ROWS      4
#define KEYPAD_COLS      4

int codeInput[] = {0, 0, 0, 0};
int codeNo = 0;
int codeOK = 0;
unsigned long lastCodeTime = 0;

char keymap[KEYPAD_ROWS][KEYPAD_COLS]={
{'1', '2', '3', 'A'},
{'4', '5', '6', 'B'},
{'7', '8', '9', 'C'},
{'*', '0', '#', 'D'}
};

byte rowPins[KEYPAD_ROWS] = {9,8,7,6}; //Rows 0 to 3
byte colPins[KEYPAD_COLS]= {5,4,3,2}; //Columns 0 to 3

//initializes an instance of the Keypad class
Keypad myKeypad= Keypad(makeKeymap(keymap), rowPins, colPins, KEYPAD_ROWS, KEYPAD_COLS);

void setup() {
  pinMode(ON_BUTTON_PIN, INPUT_PULLUP);
  pinMode(RELAY_PIN, OUTPUT);
  pinMode(CODE_WRONG_LED_PIN, OUTPUT);
  codeNo = 0;
  codeOK = 0;
  lastCodeTime = 0;

  digitalWrite(CODE_WRONG_LED_PIN, HIGH);

  Serial.begin(115200);
}

void loop(){
  char keypressed = myKeypad.getKey();
  if (keypressed != NO_KEY){
    lastCodeTime = millis();
    Serial.print(keypressed);
    codeOK = 0;
    digitalWrite(CODE_WRONG_LED_PIN, HIGH);

    codeInput[codeNo] = keypressed;
    codeNo++;
    if(codeNo == 4){
      codeNo = 0;
      if(codeInput[0] == 'D' &&
         codeInput[1] == '9' &&
         codeInput[2] == '5' &&
         codeInput[3] == '1'){
           codeOK = 1;
           digitalWrite(CODE_WRONG_LED_PIN, LOW);
           Serial.println("Code OK");
         }
         else{
            Serial.println("Code WRONG");
         }

    }
  }
  if(millis() - lastCodeTime > 5000){//Reset codeNo if nothing happened for 5000ms = 5s
    codeNo = 0;
  }
  if(digitalRead(ON_BUTTON_PIN)){
    if(codeOK){
      digitalWrite(RELAY_PIN, LOW);
      delay(200);//Wait for contact bounce to settle
    }
  }
  else{
    if(codeOK){
      digitalWrite(RELAY_PIN, HIGH);
      delay(200);//Wait for contact bounce to settle
    }
  }
}
